from classytags.arguments import Argument
from classytags.core import Tag, Options
from django import template
from django.utils.safestring import mark_safe
from cms.plugin_rendering import render_plugins
from cms.plugins.utils import get_plugins
from stacks import models as stack_models
from stacks.models import Stack

register = template.Library()


class StackNode(Tag):
    name = 'stack'
    options = Options(
        Argument('code', required=True),
        'as',
        Argument('varname', required=False, resolve=False)
    )

    def render_tag(self, context, code, varname):
        # TODO: language override (the reason this is not implemented, is that language selection is buried way
        #       down somewhere in some method called in render_plugins. There it gets extracted from the request
        #       and a language in request.GET always overrides everything.)
        if not code:
            # an empty string was passed in or the variable is not available in the context
            return ''
        # TODO: caching?
        if isinstance(code, Stack):
            stack = code
        else:
            stack, __ = stack_models.Stack.objects.get_or_create(code=code, defaults={'name': code})
        # TODO: once we drop 2.3.x support we can just use the "render_plugin" templatetag
        #       instead of rendering html here.
        placeholder = stack.content
        plugins = get_plugins(context['request'], placeholder)
        processors = ()
        rendered_placeholder = mark_safe("".join(render_plugins(plugins, context, placeholder, processors)))
        if varname:
            context[varname] = rendered_placeholder
            rendered_placeholder = u''
        return rendered_placeholder

register.tag(StackNode)



from django.utils.safestring import mark_safe
from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase
from django.utils.translation import ugettext_lazy as _
from cms.plugin_rendering import render_plugins
from stacks.fields import StackSearchField
from stacks.models import StackLink, Stack
from cms.plugins.utils import get_plugins


class StackPlugin(CMSPluginBase):
    model = StackLink
    name = _("Stack")
    render_template = "cms/plugins/stacks.html"
    admin_preview = False

    def render(self, context, instance, placeholder):
        # TODO: once we drop 2.3.x support we can just use the "render_plugin" templatetag
        #       instead of rendering html here.
        plugins = get_plugins(context['request'], instance.stack.content)
        processors = ()
        html_content = mark_safe(u"".join(render_plugins(plugins, context, placeholder, processors)))
        context.update({
            'instance': instance,
            'placeholder': placeholder,
            'content': html_content,
        })
        return context

    def formfield_for_dbfield(self, db_field, request=None, **kwargs):
        if db_field.name == "stack":
            return StackSearchField(**kwargs)
        return super(StackPlugin, self).formfield_for_dbfield(db_field, request=None, **kwargs)


plugin_pool.register_plugin(StackPlugin)



from cms.utils import get_language_from_request
from django.contrib import admin
from stacks.models import Stack
from cms.admin.placeholderadmin import PlaceholderAdmin
from django.conf import settings


class StackAdmin(PlaceholderAdmin):
    list_display = ('name', 'code',)
    search_fields = ('name', 'code',)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = extra_context or {}
        extra_context.update(self.language_tab_context(request))
        tab_language = request.GET.get("language", None)
        response = super(StackAdmin, self).change_view(request, object_id, extra_context=extra_context)

        if tab_language and response.status_code == 302 and response._headers['location'][1] == request.path:
            location = response._headers['location']
            response._headers['location'] = (location[0], "%s?language=%s" % (location[1], tab_language))
        return response

    def language_tab_context(self, request):
        language = get_language_from_request(request)
        languages = [(lang, lang_name) for lang, lang_name in settings.LANGUAGES]
        context = {
            'language': language,
            'language_tabs': languages,
            'show_language_tabs': len(languages) > 1,
        }
        return context

    def placeholder_plugin_filter(self, request, queryset):
        language = get_language_from_request(request)
        return queryset.filter(language=language)

admin.site.register(Stack, StackAdmin)



import uuid
from django.db import models
from cms.models.fields import PlaceholderField
from django.utils.translation import ugettext_lazy as _
from cms.models.pluginmodel import CMSPlugin


class Stack(models.Model):
    name = models.CharField(
        verbose_name=_(u'stack name'), max_length=255, blank=True, default='',
        help_text=_(u'Descriptive name to identify this stack. Not displayed to users.'))
    code = models.CharField(
        verbose_name=_(u'stack code'), max_length=255, unique=True, blank=True,
        help_text=_(u'To render the stack in templates.'))
    content = PlaceholderField(
        slotname=u'stack_content', verbose_name=_(u'stack content'), related_name='stacks_contents')

    class Meta:
        verbose_name = _(u'stack')
        verbose_name_plural = _(u'stacks')

    def __unicode__(self):
        return self.name

    def clean(self):
        # TODO: check for clashes if the random code is already taken
        if not self.code:
            self.code = u'stack-%s' % uuid.uuid4()


class StackLink(CMSPlugin):
    stack = models.ForeignKey(Stack, verbose_name=_(u'stack'))

    def __unicode__(self):
        return self.stack.name



# -*- coding: utf-8 -*-
from django_select2.fields import AutoModelSelect2Field
from stacks.models import Stack


class StackSearchField(AutoModelSelect2Field):
    search_fields = ('name__icontains', 'code__icontains',)
    queryset = Stack.objects



from setuptools import setup, find_packages


setup(
    name = "djangocms-stacks",
    version = __import__('stacks').__version__,
    url = 'http://github.com/divio/djangocms-stacks',
    license = 'BSD',
    platforms=['OS Independent'],
    description = "Stacks: Re-usable content blocks in django CMS.",
    long_description = open('README.rst').read() + u'\n' + open('HISTORY.rst').read(),
    author = 'Divio AG',
    author_email = 'developers@divio.ch',
    packages=find_packages(),
    install_requires = (
        # 'Django>=1.3,<1.5',  # no need to limit while in development
        'Django>=1.3',
        'django-cms>=2.3',
        'Django-Select2',
    ),
    include_package_data=True,
    zip_safe=False,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Stack'
        db.create_table('stacks_stack', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True)),
            ('code', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255, blank=True)),
            ('content', self.gf('django.db.models.fields.related.ForeignKey')(related_name='stacks_contents', null=True, to=orm['cms.Placeholder'])),
        ))
        db.send_create_signal('stacks', ['Stack'])

        # Adding model 'StackLink'
        db.create_table('cmsplugin_stacklink', (
            ('cmsplugin_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['cms.CMSPlugin'], unique=True, primary_key=True)),
            ('stack', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['stacks.Stack'])),
        ))
        db.send_create_signal('stacks', ['StackLink'])


    def backwards(self, orm):
        # Deleting model 'Stack'
        db.delete_table('stacks_stack')

        # Deleting model 'StackLink'
        db.delete_table('cmsplugin_stacklink')


    models = {
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 12, 10, 0, 0)'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.CMSPlugin']", 'null': 'True', 'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        },
        'stacks.stack': {
            'Meta': {'object_name': 'Stack'},
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255', 'blank': 'True'}),
            'content': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'stacks_contents'", 'null': 'True', 'to': "orm['cms.Placeholder']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'})
        },
        'stacks.stacklink': {
            'Meta': {'object_name': 'StackLink', 'db_table': "'cmsplugin_stacklink'", '_ormbases': ['cms.CMSPlugin']},
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'stack': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['stacks.Stack']"})
        }
    }

    complete_apps = ['stacks']